function [ pbs ] = mvnMixtureProb( x, mu, sigma, p )
% accepts: mu - NxK, a matrix of means, one column for each element in the
% mixture of normals distribution.  sigma - (N^2)xk, a matrix of covariances, each column
% should be of size NxN and contain the entries in the covariance matrix.
% df - Kx1, the degress of freedom of the multivariate t distributions.
% p - Kx1, the prior probabilities for assignment in each of the elements in the
% mixture.  x - NxP, the data points, one column for each observation.  An
% optional parameter, dimkp - NxK, lists the dimensions to disregard for
% each mixture component.
% Return value is idx - Px1 list of group assignments.

if(size(p,1)==1)
    p=p';
end;

[N P]=size(x);
[N K]=size(mu);
ll=zeros(K,P);
sig=zeros(N,N);
idx=zeros(P,1);

for(j=1:K)
    sig(:)=sigma(:,j);
    ll(j,:)=logmvnpdf(x,mu(:,j),sig);
end;
pbs=(exp(ll-repmat(max(ll),size(ll,1),1)).*repmat(p,1,size(ll,2)))';
pbs=pbs./repmat(sum(pbs,2),1,size(pbs,2));
